# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions.tests.base import BaseTestCase
from collective.eeafaceted.batchactions.utils import active_labels
from ftw.labels.interfaces import ILabeling
from ftw.labels.interfaces import ILabelJar
from ftw.labels.interfaces import ILabelRoot
from ftw.labels.interfaces import ILabelSupport
from plone import api
from zope.interface import alsoProvides
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME


class TestLabels(BaseTestCase):

    def setUp(self):
        """ """
        super(TestLabels, self).setUp()
        self.doc1 = api.content.create(self.portal, 'Document', 'doc1')
        self.doc2 = api.content.create(self.portal, 'Document', 'doc2')
        # defined some labels
        alsoProvides(self.portal, ILabelRoot)
        adapted = ILabelJar(self.portal)
        adapted.add('Pers1', 'green', True)  # label_id = pers1
        adapted.add('Pers2', 'green', True)  # label_id = pers2
        adapted.add('Pers3', 'green', True)  # label_id = pers3
        adapted.add('Glob1', 'red', False)  # label_id = glob1
        adapted.add('Glob2', 'red', False)  # label_id = glob2
        adapted.add('Glob3', 'red', False)  # label_id = glob3
        # can label created objects
        for obj in (self.doc1, self.doc2):
            alsoProvides(obj, ILabelSupport)
        self.lab_doc1 = ILabeling(self.doc1)
        self.lab_doc2 = ILabeling(self.doc2)
        login(self.portal, TEST_USER_NAME)

    def test_LabelsBatchActionForm_apply(self):
        # set 'uids' in form
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('labels-batch-action')
        form.update()
        # labels are found
        self.assertListEqual(
            form.widgets['added_values'].terms.terms.by_value.keys(),
            ['pers3:', 'pers2:', 'pers1:', 'glob3', 'glob2', 'glob1'])
        # no assigned label
        self.assertTupleEqual(active_labels(self.lab_doc1), ([], []))
        self.assertTupleEqual(active_labels(self.lab_doc2), ([], []))

        # test add action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'add',
                                                  'added_values': ['pers1:', 'glob1']}, [])
        form.handleApply(form, None)
        self.assertTupleEqual(active_labels(self.lab_doc1), (['pers1'], ['glob1']))
        self.assertTupleEqual(active_labels(self.lab_doc2), (['pers1'], ['glob1']))
        # add another pers label
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'add',
                                                  'added_values': ['pers2:']}, [])
        form.handleApply(form, None)
        act_lab = active_labels(self.lab_doc1)
        self.assertSetEqual(set(act_lab[0]), set(['pers1', 'pers2']))
        self.assertSetEqual(set(act_lab[1]), set(['glob1']))
        # add another glob label
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'add',
                                                  'added_values': ['pers2:', 'glob1', 'glob2']}, [])
        form.handleApply(form, None)
        act_lab = active_labels(self.lab_doc1)
        self.assertSetEqual(set(act_lab[0]), set(['pers1', 'pers2']))
        self.assertSetEqual(set(act_lab[1]), set(['glob1', 'glob2']))

        # test remove action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'remove',
                                                  'removed_values': ['pers1:', 'glob1']}, [])
        form.handleApply(form, None)
        act_lab = active_labels(self.lab_doc1)
        self.assertSetEqual(set(act_lab[0]), set(['pers2']))
        self.assertSetEqual(set(act_lab[1]), set(['glob2']))
        # remove all
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'remove',
                                                  'removed_values': ['pers1:', 'pers2:', 'glob1', 'glob2']}, [])
        form.handleApply(form, None)
        self.assertTupleEqual(active_labels(self.lab_doc1), ([], []))

        # test replace action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'replace',
                                                  'removed_values': ['pers1:', 'glob1'],
                                                  'added_values': ['pers2:', 'pers3:', 'glob2', 'glob3']}, [])
        form.handleApply(form, None)
        act_lab = active_labels(self.lab_doc1)
        self.assertSetEqual(set(act_lab[0]), set(['pers2', 'pers3']))
        self.assertSetEqual(set(act_lab[1]), set(['glob2', 'glob3']))

        # test overwrite action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'overwrite',
                                                  'added_values': ['pers1:', 'glob1']}, [])
        form.handleApply(form, None)
        act_lab = active_labels(self.lab_doc1)
        self.assertSetEqual(set(act_lab[0]), set(['pers1']))
        self.assertSetEqual(set(act_lab[1]), set(['glob1']))



# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions.tests.base import BaseTestCase
from collective.eeafaceted.batchactions.utils import brains_from_uids
from collective.eeafaceted.batchactions.utils import filter_on_permission
from collective.eeafaceted.batchactions.utils import has_interface
from collective.eeafaceted.batchactions.utils import is_permitted
from ftw.labels.interfaces import ILabelSupport
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from zope.interface import alsoProvides


class TestUtils(BaseTestCase):

    def setUp(self):
        """ """
        super(TestUtils, self).setUp()
        self.doc1 = api.content.create(
            type='Document',
            id='doc1',
            title='Document 1',
            container=self.portal
        )
        self.doc2 = api.content.create(
            type='Document',
            id='doc2',
            title='Document 2',
            container=self.portal
        )
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Member'])

    def test_filter_on_permission(self):
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        brains = brains_from_uids(doc_uids)
        self.assertEquals(len(filter_on_permission(brains)), 2)
        self.assertEquals(len(filter_on_permission(brains, 'Review comments')), 0)
        setRoles(self.portal, TEST_USER_ID, ['Member', 'Reviewer'])
        self.assertEquals(len(filter_on_permission(brains, 'Review comments')), 2)

    def test_is_permitted(self):
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        brains = brains_from_uids(doc_uids)
        self.assertTrue(is_permitted(brains))
        self.assertFalse(is_permitted(brains, 'Review comments'))

    def test_has_interface(self):
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        brains = brains_from_uids(doc_uids)
        self.assertFalse(has_interface(brains, ILabelSupport))
        alsoProvides(self.doc1, ILabelSupport)
        self.assertFalse(has_interface(brains, ILabelSupport))
        alsoProvides(self.doc2, ILabelSupport)
        self.assertTrue(has_interface(brains, ILabelSupport))

    def test_brains_from_uids(self):
        self.assertEqual(len(brains_from_uids('')), 0)
        self.assertEqual(len(brains_from_uids('{},{}'.format(self.doc1.UID(), self.doc2.UID()))), 2)
        self.assertEqual(len(brains_from_uids([self.doc1.UID(), self.doc2.UID()])), 2)



# -*- coding: utf-8 -*-
"""Batch actions views."""

from AccessControl import Unauthorized
from collective.eeafaceted.batchactions import _
from collective.eeafaceted.batchactions.utils import active_labels
from collective.eeafaceted.batchactions.utils import brains_from_uids
from collective.eeafaceted.batchactions.utils import cannot_modify_field_msg
from collective.eeafaceted.batchactions.utils import has_interface
from collective.eeafaceted.batchactions.utils import is_permitted
from operator import attrgetter
from plone import api
from plone.formwidget.masterselect import MasterSelectField
from plone.supermodel import model
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.CMFPlone.utils import safe_unicode
from z3c.form import button
from z3c.form.browser.select import SelectFieldWidget
from z3c.form.field import Fields
from z3c.form.form import Form
from z3c.form.interfaces import HIDDEN_MODE
from zope import schema
from zope.component import getUtility
from zope.i18n import translate
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import modified
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class IBaseBatchActionsFormSchema(model.Schema):

    uids = schema.TextLine(
        title=u"uids",
        description=u''
    )

    referer = schema.TextLine(
        title=u'referer',
        required=False,
    )


class BaseBatchActionForm(Form):

    label = _(u"Batch action form")
    fields = Fields(IBaseBatchActionsFormSchema)
    fields['uids'].mode = HIDDEN_MODE
    fields['referer'].mode = HIDDEN_MODE
    ignoreContext = True
    brains = []
    do_apply = True
    # this will add a specific class to the generated button action
    # so it is possible to skin it with an icon
    button_with_icon = False
    overlay = True
    weight = 100

    def available(self):
        """Will the action be available for current context?"""
        return True

    def _update(self):
        """Method to override if you need to do something in the update."""
        return

    def _update_widgets(self):
        """Method to override if you need to do something after the updateWidgets method."""
        return

    @property
    def description(self):
        """ """
        # update description depending on number of brains
        return _('This action will affect ${number} element(s).',
                 mapping={'number': len(self.brains)})

    def _apply(self, **data):
        """This method receives in data the form content and does the apply logic.
           It is the method to implement if default handleApply is enough."""
        raise NotImplementedError

    def update(self):
        form = self.request.form
        if 'form.widgets.uids' in form:
            uids = form['form.widgets.uids']
        else:
            uids = self.request.get('uids', '')
            form['form.widgets.uids'] = uids

        if 'form.widgets.referer' not in form:
            form['form.widgets.referer'] = self.request.get('referer', '').replace('@', '&').replace('!', '#')

        self.brains = self.brains or brains_from_uids(uids)

        # sort buttons
        self._old_buttons = self.buttons
        self.buttons = self.buttons.select('apply', 'cancel')
        self._update()
        super(BaseBatchActionForm, self).update()
        self._update_widgets()

    @button.buttonAndHandler(_(u'Apply'), name='apply', condition=lambda fi: fi.do_apply)
    def handleApply(self, action):
        """ """
        if not self.available():
            raise Unauthorized

        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
        else:
            # call the method that does the job
            self._apply(**data)
            self.request.response.redirect(self.request.form['form.widgets.referer'])

    @button.buttonAndHandler(PMF(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        self.request.response.redirect(self.request.get('HTTP_REFERER'))


class TransitionBatchActionForm(BaseBatchActionForm):

    label = _(u"Batch state change")
    weight = 10

    def getAvailableTransitionsVoc(self):
        """ Returns available transitions common for all brains """
        wtool = api.portal.get_tool(name='portal_workflow')
        terms = []
        transitions = None
        for brain in self.brains:
            obj = brain.getObject()
            if transitions is None:
                transitions = set([(tr['id'], tr['title']) for tr in wtool.getTransitionsFor(obj)])
            else:
                transitions &= set([(tr['id'], tr['title']) for tr in wtool.getTransitionsFor(obj)])
        if transitions:
            for (id, tit) in transitions:
                terms.append(
                    SimpleTerm(id,
                               id,
                               translate(tit,
                                         domain='plone',
                                         context=self.request)))
        terms = sorted(terms, key=attrgetter('title'))
        return SimpleVocabulary(terms)

    def _update(self):
        self.voc = self.getAvailableTransitionsVoc()
        self.do_apply = len(self.voc) > 0
        self.fields += Fields(schema.Choice(
            __name__='transition',
            title=_(u'Transition'),
            vocabulary=self.voc,
            description=(len(self.voc) == 0 and
                         _(u'No common or available transition. Modify your selection.') or u''),
            required=len(self.voc) > 0))
        self.fields += Fields(schema.Text(
            __name__='comment',
            title=_(u'Comment'),
            description=_(u'Optional comment to display in history'),
            required=False))

    def _apply(self, **data):
        """ """
        if data['transition']:
            for brain in self.brains:
                obj = brain.getObject()
                api.content.transition(obj=obj,
                                       transition=data['transition'],
                                       comment=data['comment'])


try:
    from ftw.labels.interfaces import ILabeling
    from ftw.labels.interfaces import ILabelJar
    from ftw.labels.interfaces import ILabelSupport
except ImportError:
    pass


class LabelsBatchActionForm(BaseBatchActionForm):

    label = _(u"Batch labels change")
    weight = 20

    def get_labeljar_context(self):
        return self.context

    def get_labels_vocabulary(self):
        terms, p_labels, g_labels = [], [], []
        context = self.get_labeljar_context()
        try:
            adapted = ILabelJar(context)
        except:
            return SimpleVocabulary(terms), [], []
        self.can_change_labels = is_permitted(self.brains, perm='ftw.labels: Change Labels')
        for label in adapted.list():
            if label['by_user']:
                p_labels.append(label['label_id'])
                terms.append(SimpleVocabulary.createTerm('%s:' % label['label_id'],
                                                         label['label_id'],
                                                         u'{} (*)'.format(safe_unicode(label['title']))))
            else:
                g_labels.append(label['label_id'])
                if self.can_change_labels:
                    terms.append(SimpleVocabulary.createTerm(label['label_id'], label['label_id'],
                                                             safe_unicode(label['title'])))
        return SimpleVocabulary(terms), set(p_labels), g_labels

    def _update(self):
        labels_voc, self.p_labels, self.g_labels = self.get_labels_vocabulary()
        self.do_apply = len(labels_voc._terms) and has_interface(self.brains, ILabelSupport)
        self.fields += Fields(MasterSelectField(
            __name__='action_choice',
            title=_(u'Batch action choice'),
            description=(not self.do_apply and cannot_modify_field_msg or u''),
            vocabulary=SimpleVocabulary([SimpleTerm(value=u'add', title=_(u'Add items')),
                                         SimpleTerm(value=u'remove', title=_(u'Remove items')),
                                         SimpleTerm(value=u'replace', title=_(u'Replace some items by others')),
                                         SimpleTerm(value=u'overwrite', title=_(u'Overwrite'))]),
            slave_fields=(
                {'name': 'removed_values',
                 'slaveID': '#form-widgets-removed_values',
                 'action': 'hide',
                 'hide_values': (u'add', u'overwrite'),
                 'siblings': True,
                 },
                {'name': 'added_values',
                 'slaveID': '#form-widgets-added_values',
                 'action': 'hide',
                 'hide_values': (u'remove'),
                 'siblings': True,
                 },
            ),
            required=self.do_apply,
            default=u'add'
        ))
        if self.do_apply:
            self.fields += Fields(schema.List(
                __name__='removed_values',
                title=_(u"Removed values"),
                description=_(u"Select the values to remove (CTRL+click). A personal label is represented by (*)."),
                required=False,
                value_type=schema.Choice(vocabulary=labels_voc),
            ))
            self.fields += Fields(schema.List(
                __name__='added_values',
                title=_(u"Added values"),
                description=_(u"Select the values to add (CTRL+click). A personal label is represented by (*)."),
                required=False,
                value_type=schema.Choice(vocabulary=labels_voc),
            ))
            self.fields["removed_values"].widgetFactory = SelectFieldWidget
            self.fields["added_values"].widgetFactory = SelectFieldWidget

    def _update_widgets(self):
        if self.do_apply:
            #        self.widgets['action_choice'].size = 4
            self.widgets['removed_values'].multiple = 'multiple'
            self.widgets['removed_values'].size = 5
            self.widgets['added_values'].multiple = 'multiple'
            self.widgets['added_values'].size = 5

    def _apply(self, **data):
        if ((data.get('removed_values', None) and data['action_choice'] in ('remove', 'replace')) or
           (data.get('added_values', None)) and data['action_choice'] in ('add', 'replace', 'overwrite')):
            values = {'p_a': [], 'p_r': [], 'g_a': [], 'g_r': []}
            for act, lst in (('a', data.get('added_values', [])), ('r', data.get('removed_values', []))):
                for val in lst:
                    typ = (':' in val) and 'p' or 'g'
                    values['{}_{}'.format(typ, act)].append(val.split(':')[0])
            for brain in self.brains:
                obj = brain.getObject()
                labeling = ILabeling(obj)
                p_act, g_act = active_labels(labeling)
                # manage global labels
                if self.can_change_labels and (values['g_a'] or values['g_r']):
                    if data['action_choice'] in ('overwrite'):
                        items = set(values['g_a'])
                    else:
                        items = set(g_act)  # currently active labels
                        if data['action_choice'] in ('remove', 'replace'):
                            items = items.difference(values['g_r'])
                        if data['action_choice'] in ('add', 'replace'):
                            items = items.union(values['g_a'])
                    labeling.update(items)
                # manage personal labels
                if values['p_a'] or values['p_r']:
                    if data['action_choice'] in ('overwrite'):
                        items = set(values['p_a'])
                        labeling.pers_update(self.p_labels.difference(items), False)
                        labeling.pers_update(items, True)
                    else:
                        if data['action_choice'] in ('remove', 'replace'):
                            labeling.pers_update(set(p_act).intersection(values['p_r']), False)
                        if data['action_choice'] in ('add', 'replace'):
                            labeling.pers_update(values['p_a'], True)
                obj.reindexObject(['labels'])

try:
    from collective.contact.widget.schema import ContactList
    from z3c.relationfield.relation import RelationValue
except ImportError:
    pass


class ContactBaseBatchActionForm(BaseBatchActionForm):
    """
        Base class to manage contact field change.
        For now, only ContactList.
    """

    label = _(u"Batch contact field change")
    weight = 30
    # Following variables must be overrided in child class
    available_permission = ''
    attribute = ''
    field_value_type = None

    def available(self):
        """Will the action be available for current context?"""
        if self.available_permission:
            return api.user.has_permission(self.available_permission)
        return True

    def _update(self):
        assert self.attribute
        assert self.field_value_type is not None
        self.do_apply = is_permitted(self.brains)
        self.fields += Fields(schema.Choice(
            __name__='action_choice',
            title=_(u'Batch action choice'),
            description=(not self.do_apply and cannot_modify_field_msg or u''),
            vocabulary=SimpleVocabulary([SimpleTerm(value=u'add', title=_(u'Add items')),
                                         SimpleTerm(value=u'remove', title=_(u'Remove items')),
                                         SimpleTerm(value=u'replace', title=_(u'Replace some items by others')),
                                         SimpleTerm(value=u'overwrite', title=_(u'Overwrite'))]),
            required=self.do_apply,
            default=u'add'
        ))
        if self.do_apply:
            self.fields += Fields(ContactList(
                __name__='removed_values',
                title=_(u"Removed values"),
                description=_(u"Search and select the values to remove, if necessary."),
                required=False,
                addlink=False,
                value_type=self.field_value_type,
            ))
            self.fields += Fields(ContactList(
                __name__='added_values',
                title=_(u"Added values"),
                description=_(u"Search and select the values to add."),
                required=False,
                addlink=False,
                value_type=self.field_value_type,
            ))

    def _apply(self, **data):
        if ((data.get('removed_values', None) and data['action_choice'] in ('remove', 'replace')) or
           (data.get('added_values', None)) and data['action_choice'] in ('add', 'replace', 'overwrite')):
            intids = getUtility(IIntIds)
            for brain in self.brains:
                obj = brain.getObject()
                if data['action_choice'] in ('overwrite'):
                    items = set(data['added_values'])
                else:
                    # we get the linked objects
                    items = set([intids.getObject(rel.to_id) for rel in (getattr(obj, self.attribute) or [])
                                 if not rel.isBroken()])
                    if data['action_choice'] in ('remove', 'replace'):
                        items = items.difference(data['removed_values'])
                    if data['action_choice'] in ('add', 'replace'):
                        items = items.union(data['added_values'])
                # transform to relations
                rels = [RelationValue(intids.getId(ob)) for ob in items]
                setattr(obj, self.attribute, rels)
                modified(obj)



# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions.tests.base import BaseTestCase
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from z3c.relationfield.relation import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


class TestContactForm(BaseTestCase):

    def setUp(self):
        """ """
        super(TestContactForm, self).setUp()
        self.typ1 = api.content.create(self.eea_folder, 'testtype', 'typ1')
        self.typ2 = api.content.create(self.eea_folder, 'testtype', 'typ2')
        org1 = self.portal['mydirectory']['armeedeterre']
        org2 = org1['corpsa']
        org3 = org1['corpsb']
        self.intids = getUtility(IIntIds)
        self.orgs = [org1, org2, org3]
        self.rels = [RelationValue(self.intids.getId(org1)), RelationValue(self.intids.getId(org2)),
                     RelationValue(self.intids.getId(org3))]
        # set 'uids' in form
        typ_uids = u"{0},{1}".format(self.typ1.UID(), self.typ2.UID())
        self.request.form['form.widgets.uids'] = typ_uids

    def to_objs(self, rels):
        return sorted([self.intids.getObject(rel.to_id) for rel in rels or []], key=lambda x: self.orgs.index(x))

    def test_ContactBatchActionForm_available(self):
        login(self.portal, TEST_USER_NAME)
        form = self.eea_folder.restrictedTraverse('contact-batch-action')
        form.update()
        self.assertIn('Manager', api.user.get_roles())
        self.assertTrue(form.available())
        setRoles(self.portal, TEST_USER_ID, [])
        self.assertNotIn('Manager', api.user.get_roles())
        self.assertFalse(form.available())

    def test_ContactBatchActionForm_apply(self):
        form = self.eea_folder.restrictedTraverse('contact-batch-action')
        form.update()
        # test add action on None attributes
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'add',
                                                  'added_values': self.orgs[0:1]}, [])
        form.handleApply(form, None)
        self.assertEqual(self.to_objs(self.typ1.related_organizations), self.orgs[0:1])
        self.assertEqual(self.to_objs(self.typ2.related_organizations), self.orgs[0:1])
        # set values
        self.typ1.related_organizations = self.rels[0:2]
        self.typ2.related_organizations = self.rels[1:3]
        # test add action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'add',
                                                  'added_values': self.orgs[0:1]}, [])
        form.handleApply(form, None)
        self.assertEqual(self.to_objs(self.typ1.related_organizations), self.orgs[0:2])
        self.assertEqual(self.to_objs(self.typ2.related_organizations), self.orgs)
        # test remove action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'remove',
                                                  'removed_values': self.orgs[0:2]}, [])
        form.handleApply(form, None)
        self.assertEqual(self.to_objs(self.typ1.related_organizations), [])
        self.assertEqual(self.to_objs(self.typ2.related_organizations), self.orgs[2:3])
        # test replace action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'replace',
                                                  'removed_values': self.orgs[1:3],
                                                  'added_values': self.orgs[0:2]}, [])
        form.handleApply(form, None)
        self.assertEqual(self.to_objs(self.typ1.related_organizations), self.orgs[0:2])
        self.assertEqual(self.to_objs(self.typ2.related_organizations), self.orgs[0:2])
        # test overwrite action
        form.widgets.extract = lambda *a, **kw: ({'action_choice': 'overwrite',
                                                  'added_values': self.orgs[1:3]}, [])
        form.handleApply(form, None)
        self.assertEqual(self.to_objs(self.typ1.related_organizations), self.orgs[1:3])
        self.assertEqual(self.to_objs(self.typ2.related_organizations), self.orgs[1:3])



# -*- coding: utf-8 -*-
"""Batch actions views."""

from AccessControl import getSecurityManager
from collective.eeafaceted.batchactions import _
from plone import api

cannot_modify_field_msg = _(u"You can't change this field on selected items. Modify your selection.")


def is_permitted(brains, perm='Modify portal content'):
    """ Check all brains to verify a permission, by default 'Modify portal content' """
    ret = True
    sm = getSecurityManager()
    for brain in brains:
        obj = brain.getObject()
        if not sm.checkPermission(perm, obj):
            ret = False
            break
    return ret


def has_interface(brains, itf):
    """ Check all brains to verify a provided interface """
    ret = True
    for brain in brains:
        obj = brain.getObject()
        if not itf.providedBy(obj):
            ret = False
            break
    return ret


def filter_on_permission(brains, perm='Modify portal content'):
    """ Return only objects where current user has the permission """
    ret = []
    sm = getSecurityManager()
    for brain in brains:
        obj = brain.getObject()
        if sm.checkPermission(perm, obj):
            ret.append(obj)
    return ret


def listify_uids(uids):
    """ uids is received as a string separated by commas, turn it into a real list """
    if isinstance(uids, basestring):
        uids = uids.split(',')
    return uids


def brains_from_uids(uids):
    """ Returns a list of brains from a string (comma separated) or a list, containing uids """
    if not uids:
        return []

    catalog = api.portal.get_tool('portal_catalog')
    uids = listify_uids(uids)
    brains = catalog(UID=uids)
    return brains


def active_labels(labeling):
    """
        For ftw.labels only.
        Returns 2 list of active labels on an adapted object : personal and global
    """
    p_act, g_act = [], []
    for label_id in labeling.storage:
        try:
            label = labeling.jar.get(label_id)
            if label['by_user']:
                if labeling.user_id() in labeling.storage[label_id]:
                    p_act.append(label_id)
            else:
                g_act.append(label_id)
        except KeyError:
            pass
    return p_act, g_act
